from typing import List, Optional, Type, cast
from xml.etree.ElementTree import Element, parse

from py_pdf_term._common.data import Term
from py_pdf_term.pdftoxml import PDFnXMLElement, PDFnXMLPath
from py_pdf_term.tokenizer import Token, Tokenizer
from py_pdf_term.tokenizer import BaseLanguageTokenizer

from .augmenters import AugmenterCombiner, BaseAugmenter
from .classifiers import BaseTokenClassifier, MeaninglessMarker
from .data import DomainCandidateTermList, PageCandidateTermList, PDFCandidateTermList
from .filters import BaseCandidateTermFilter, BaseCandidateTokenFilter, FilterCombiner
from .splitters import BaseSplitter, SplitterCombiner
from .utils import textnode_fontsize, textnode_ncolor, textnode_text


class CandidateTermExtractor:
    def __init__(
        self,
        lang_tokenizer_clses: Optional[List[Type[BaseLanguageTokenizer]]] = None,
        token_classifier_clses: Optional[List[Type[BaseTokenClassifier]]] = None,
        token_filter_clses: Optional[List[Type[BaseCandidateTokenFilter]]] = None,
        term_filter_clses: Optional[List[Type[BaseCandidateTermFilter]]] = None,
        splitter_clses: Optional[List[Type[BaseSplitter]]] = None,
        augmenter_clses: Optional[List[Type[BaseAugmenter]]] = None,
    ) -> None:
        lang_tokenizers = (
            list(map(lambda cls: cls(), lang_tokenizer_clses))
            if lang_tokenizer_clses is not None
            else None
        )
        self._tokenizer = Tokenizer(lang_tokenizers=lang_tokenizers)

        token_filters = (
            list(map(lambda cls: cls(), token_filter_clses))
            if token_filter_clses is not None
            else None
        )
        term_filters = (
            list(map(lambda cls: cls(), term_filter_clses))
            if term_filter_clses is not None
            else None
        )
        self._filter = FilterCombiner(token_filters, term_filters)

        classifiers = (
            list(map(lambda cls: cls(), token_classifier_clses))
            if token_classifier_clses is not None
            else None
        )
        self._marker = MeaninglessMarker(classifiers)

        splitters = (
            list(map(lambda cls: cls(classifiers), splitter_clses))
            if splitter_clses is not None
            else None
        )
        self._splitter = SplitterCombiner(splitters)

        augmenters = (
            list(map(lambda cls: cls(), augmenter_clses))
            if augmenter_clses is not None
            else None
        )
        self._augmenter = AugmenterCombiner(augmenters)

    def extract_from_domain_files(
        self, domain: str, pdfnxmls: List[PDFnXMLPath]
    ) -> DomainCandidateTermList:
        xmls = list(map(self.extract_from_xml_file, pdfnxmls))
        return DomainCandidateTermList(domain, xmls)

    def extract_from_xml_file(self, pdfnxml: PDFnXMLPath) -> PDFCandidateTermList:
        xml_root = parse(pdfnxml.xml_path).getroot()
        xml_candidates = self._extract_from_xmlroot(pdfnxml.pdf_path, xml_root)
        return xml_candidates

    def extract_from_domain_elements(
        self, domain: str, pdfnxmls: List[PDFnXMLElement]
    ) -> DomainCandidateTermList:
        xmls = list(map(self.extract_from_xml_element, pdfnxmls))
        return DomainCandidateTermList(domain, xmls)

    def extract_from_xml_element(self, pdfnxml: PDFnXMLElement) -> PDFCandidateTermList:
        xml_candidates = self._extract_from_xmlroot(pdfnxml.pdf_path, pdfnxml.xml_root)
        return xml_candidates

    def extract_from_text(
        self, text: str, fontsize: float = 0.0, ncolor: str = ""
    ) -> List[Term]:
        tokens = self._tokenizer.tokenize(text)
        return self._extract_from_tokens(tokens, fontsize, ncolor)

    def _extract_from_xmlroot(
        self, pdf_path: str, xml_root: Element
    ) -> PDFCandidateTermList:
        page_candidates: List[PageCandidateTermList] = []
        for page in xml_root.iter("page"):
            page_candidates.append(self._extract_from_page(page))

        return PDFCandidateTermList(pdf_path, page_candidates)

    def _extract_from_page(self, page: Element) -> PageCandidateTermList:
        page_num = int(cast(str, page.get("id")))

        candicate_terms: List[Term] = []
        for textnode in page.iter("text"):
            text = textnode_text(textnode)
            fontsize = textnode_fontsize(textnode)
            ncolor = textnode_ncolor(textnode)
            tokens = self._tokenizer.tokenize(text)
            terms = self._extract_from_tokens(tokens, fontsize, ncolor)
            candicate_terms.extend(terms)

        return PageCandidateTermList(page_num, candicate_terms)

    def _extract_from_tokens(
        self, tokens: List[Token], fontsize: float, ncolor: str
    ) -> List[Term]:
        candicate_terms: List[Term] = []
        candicate_tokens: List[Token] = []
        for idx, token in enumerate(tokens):
            if self._filter.is_partof_candidate(tokens, idx):
                candicate_tokens.append(token)
                continue

            terms = self._terms_from_tokens(candicate_tokens, fontsize, ncolor)
            candicate_terms.extend(terms)
            candicate_tokens = []

        terms = self._terms_from_tokens(candicate_tokens, fontsize, ncolor)
        candicate_terms.extend(terms)

        return candicate_terms

    def _terms_from_tokens(
        self, candicate_tokens: List[Token], fontsize: float, ncolor: str
    ) -> List[Term]:
        candidate_term = Term(candicate_tokens, fontsize, ncolor)

        candicate_terms: List[Term] = []
        splitted_candidates = self._splitter.split(candidate_term)
        for splitted_candidate in splitted_candidates:
            augmented_candidates = self._augmenter.augment(splitted_candidate)
            candicate_terms.extend(augmented_candidates)
            candicate_terms.append(splitted_candidate)

        return list(
            map(self._marker.mark, filter(self._filter.is_candidate, candicate_terms))
        )



import re
from dataclasses import asdict, dataclass
from typing import Any, Dict, List, Union

from py_pdf_term._common.consts import NOSPACE_REGEX
from py_pdf_term.tokenizer import Token

GARBAGE_SPACE = re.compile(rf"(?<={NOSPACE_REGEX}) (?=\S)|(?<=\S) (?={NOSPACE_REGEX})")


@dataclass(frozen=True)
class Term:
    tokens: List[Token]
    fontsize: float = 0.0
    ncolor: str = ""
    augmented: bool = False

    @property
    def lang(self) -> Union[str, None]:
        if not self.tokens:
            return None

        lang = self.tokens[0].lang
        if all(map(lambda token: token.lang == lang, self.tokens)):
            return lang

        return None

    def __str__(self) -> str:
        return GARBAGE_SPACE.sub("", " ".join(map(str, self.tokens)))

    def surface_form(self) -> str:
        return GARBAGE_SPACE.sub(
            "", " ".join(map(lambda token: token.surface_form, self.tokens))
        )

    def lemma(self) -> str:
        return GARBAGE_SPACE.sub(
            "", " ".join(map(lambda token: token.lemma, self.tokens))
        )

    def to_dict(self) -> Dict[str, Any]:
        return {
            "tokens": list(map(lambda token: token.to_dict(), self.tokens)),
            "fontsize": self.fontsize,
            "ncolor": self.ncolor,
            "augmented": self.augmented,
        }

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> "Term":
        return cls(
            list(map(lambda item: Token.from_dict(item), obj["tokens"])),
            obj.get("fontsize", 0),
            obj.get("ncolor", ""),
            obj.get("augmented", False),
        )


@dataclass(frozen=True)
class ScoredTerm:
    term: str
    score: float

    def __str__(self) -> str:
        return self.term

    def to_dict(self) -> Dict[str, Any]:
        return asdict(self)

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> "ScoredTerm":
        return cls(**obj)



from .base import BaseMultiDomainRankingMethod, BaseSingleDomainRankingMethod
from .data import MethodTermRanking
from .flr import FLRMethod
from .flrh import FLRHMethod
from .hits import HITSMethod
from .mcvalue import MCValueMethod
from .mdp import MDPMethod
from .tfidf import TFIDFMethod

# isort: unique-list
__all__ = [
    "BaseMultiDomainRankingMethod",
    "BaseSingleDomainRankingMethod",
    "FLRHMethod",
    "FLRMethod",
    "HITSMethod",
    "MCValueMethod",
    "MDPMethod",
    "MethodTermRanking",
    "TFIDFMethod",
]



from dataclasses import asdict, dataclass
from typing import Any, Dict, TypeVar


@dataclass(frozen=True)
class BaseRankingData:
    domain: str
    # unique domain name

    def to_dict(self) -> Dict[str, Any]:
        return asdict(self)

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> "BaseRankingData":
        return cls(**obj)


RankingData = TypeVar("RankingData", bound=BaseRankingData)



from .candidate import CandidateLayerCacheMapper
from .method import MethodLayerDataCacheMapper, MethodLayerRankingCacheMapper
from .styling import StylingLayerCacheMapper
from .xml import XMLLayerCacheMapper

# isort: unique-list
__all__ = [
    "CandidateLayerCacheMapper",
    "MethodLayerDataCacheMapper",
    "MethodLayerRankingCacheMapper",
    "StylingLayerCacheMapper",
    "XMLLayerCacheMapper",
]



from abc import ABCMeta, abstractmethod

from py_pdf_term._common.data import Term
from py_pdf_term.candidates import PageCandidateTermList


class BaseStylingScore(metaclass=ABCMeta):
    def __init__(self, page_candidates: PageCandidateTermList) -> None:
        pass

    @abstractmethod
    def calculate_score(self, candidate: Term) -> float:
        raise NotImplementedError(f"{self.__class__.__name__}.calculate_score()")



from .base import BaseTokenClassifier
from .english import EnglishTokenClassifier
from .japanese import JapaneseTokenClassifier
from .marker import MeaninglessMarker

# isort: unique-list
__all__ = [
    "BaseTokenClassifier",
    "EnglishTokenClassifier",
    "JapaneseTokenClassifier",
    "MeaninglessMarker",
]



from dataclasses import dataclass
from typing import Optional

from .base import BaseLayerConfig


@dataclass(frozen=True)
class XMLLayerConfig(BaseLayerConfig):
    open_bin: str = "python.open"
    include_pattern: Optional[str] = None
    exclude_pattern: Optional[str] = None
    nfc_norm: bool = True
    cache: str = "py_pdf_term.XMLLayerFileCache"



from .caches import (
    CandidateLayerCacheMapper,
    MethodLayerDataCacheMapper,
    MethodLayerRankingCacheMapper,
    StylingLayerCacheMapper,
    XMLLayerCacheMapper,
)
from .candidates import (
    AugmenterMapper,
    CandidateTermFilterMapper,
    CandidateTokenFilterMapper,
    LanguageTokenizerMapper,
    SplitterMapper,
    TokenClassifilerMapper,
)
from .methods import MultiDomainRankingMethodMapper, SingleDomainRankingMethodMapper
from .pdftoxml import BinaryOpenerMapper
from .stylings import StylingScoreMapper

# isort: unique-list
__all__ = [
    "AugmenterMapper",
    "BinaryOpenerMapper",
    "CandidateLayerCacheMapper",
    "CandidateTermFilterMapper",
    "CandidateTokenFilterMapper",
    "LanguageTokenizerMapper",
    "MethodLayerDataCacheMapper",
    "MethodLayerRankingCacheMapper",
    "MultiDomainRankingMethodMapper",
    "SingleDomainRankingMethodMapper",
    "SplitterMapper",
    "StylingLayerCacheMapper",
    "StylingScoreMapper",
    "TokenClassifilerMapper",
    "XMLLayerCacheMapper",
]



from typing import Any, Type

from py_pdf_term.methods import BaseMultiDomainRankingMethod, MDPMethod, TFIDFMethod

from ..base import BaseMapper
from ..consts import PACKAGE_NAME


class MultiDomainRankingMethodMapper(
    BaseMapper[Type[BaseMultiDomainRankingMethod[Any]]]
):
    @classmethod
    def default_mapper(cls) -> "MultiDomainRankingMethodMapper":
        default_mapper = cls()

        multi_domain_clses = [TFIDFMethod, MDPMethod]
        for method_cls in multi_domain_clses:
            default_mapper.add(f"{PACKAGE_NAME}.{method_cls.__name__}", method_cls)

        return default_mapper



from .endtoend._endtoend.mappers import (
    AugmenterMapper,
    BinaryOpenerMapper,
    CandidateLayerCacheMapper,
    CandidateTermFilterMapper,
    CandidateTokenFilterMapper,
    TokenClassifilerMapper,
    LanguageTokenizerMapper,
    MethodLayerDataCacheMapper,
    MethodLayerRankingCacheMapper,
    MultiDomainRankingMethodMapper,
    SingleDomainRankingMethodMapper,
    SplitterMapper,
    StylingLayerCacheMapper,
    StylingScoreMapper,
    XMLLayerCacheMapper,
)

# isort: unique-list
__all__ = [
    "AugmenterMapper",
    "BinaryOpenerMapper",
    "CandidateLayerCacheMapper",
    "CandidateTermFilterMapper",
    "CandidateTokenFilterMapper",
    "LanguageTokenizerMapper",
    "MethodLayerDataCacheMapper",
    "MethodLayerRankingCacheMapper",
    "MultiDomainRankingMethodMapper",
    "SingleDomainRankingMethodMapper",
    "SplitterMapper",
    "StylingLayerCacheMapper",
    "StylingScoreMapper",
    "XMLLayerCacheMapper",
    "TokenClassifilerMapper",
]



from abc import ABCMeta, abstractmethod
from typing import List

from py_pdf_term.tokenizer import Token


class BaseCandidateTokenFilter(metaclass=ABCMeta):
    def __init__(self) -> None:
        pass

    @abstractmethod
    def inscope(self, token: Token) -> bool:
        raise NotImplementedError(f"{self.__class__.__name__}.inscope()")

    @abstractmethod
    def is_partof_candidate(self, tokens: List[Token], idx: int) -> bool:
        raise NotImplementedError(f"{self.__class__.__name__}.is_partof_candidate()")



from py_pdf_term.analysis import TermOccurrenceAnalyzer
from py_pdf_term.candidates import DomainCandidateTermList

from ..rankingdata import MDPRankingData
from .base import BaseRankingDataCollector


class MDPRankingDataCollector(BaseRankingDataCollector[MDPRankingData]):
    def __init__(self) -> None:
        super().__init__()
        self._termocc_analyzer = TermOccurrenceAnalyzer()

    def collect(self, domain_candidates: DomainCandidateTermList) -> MDPRankingData:
        termocc = self._termocc_analyzer.analyze(domain_candidates)
        return MDPRankingData(domain_candidates.domain, termocc.term_freq)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['py_pdf_term',
 'py_pdf_term._common',
 'py_pdf_term.analysis',
 'py_pdf_term.analysis._analysis',
 'py_pdf_term.analysis._analysis.concats',
 'py_pdf_term.analysis._analysis.cooccurrences',
 'py_pdf_term.analysis._analysis.occurrences',
 'py_pdf_term.candidates',
 'py_pdf_term.candidates._candidates',
 'py_pdf_term.candidates._candidates.augmenters',
 'py_pdf_term.candidates._candidates.classifiers',
 'py_pdf_term.candidates._candidates.filters',
 'py_pdf_term.candidates._candidates.filters.term',
 'py_pdf_term.candidates._candidates.filters.term.concatenation',
 'py_pdf_term.candidates._candidates.filters.term.numeric',
 'py_pdf_term.candidates._candidates.filters.term.propernoun',
 'py_pdf_term.candidates._candidates.filters.term.symbollike',
 'py_pdf_term.candidates._candidates.filters.token',
 'py_pdf_term.candidates._candidates.splitters',
 'py_pdf_term.endtoend',
 'py_pdf_term.endtoend._endtoend',
 'py_pdf_term.endtoend._endtoend.caches',
 'py_pdf_term.endtoend._endtoend.caches.candidate',
 'py_pdf_term.endtoend._endtoend.caches.method',
 'py_pdf_term.endtoend._endtoend.caches.styling',
 'py_pdf_term.endtoend._endtoend.caches.xml',
 'py_pdf_term.endtoend._endtoend.configs',
 'py_pdf_term.endtoend._endtoend.layers',
 'py_pdf_term.endtoend._endtoend.mappers',
 'py_pdf_term.endtoend._endtoend.mappers.caches',
 'py_pdf_term.endtoend._endtoend.mappers.candidates',
 'py_pdf_term.endtoend._endtoend.mappers.methods',
 'py_pdf_term.endtoend._endtoend.mappers.pdftoxml',
 'py_pdf_term.endtoend._endtoend.mappers.stylings',
 'py_pdf_term.methods',
 'py_pdf_term.methods._methods',
 'py_pdf_term.methods._methods.collectors',
 'py_pdf_term.methods._methods.rankers',
 'py_pdf_term.methods._methods.rankingdata',
 'py_pdf_term.pdftoxml',
 'py_pdf_term.pdftoxml._pdftoxml',
 'py_pdf_term.stylings',
 'py_pdf_term.stylings._stylings',
 'py_pdf_term.stylings._stylings.scores',
 'py_pdf_term.techterms',
 'py_pdf_term.techterms._techterms',
 'py_pdf_term.tokenizer',
 'py_pdf_term.tokenizer._tokenizer']

package_data = \
{'': ['*']}

install_requires = \
['pdfminer.six>=20211012,<20211013', 'spacy>=3.2.3,<4.0.0']

setup_kwargs = {
    'name': 'py-pdf-term',
    'version': '0.18.1',
    'description': 'A fully-configurable terminology extraction module written in Python',
    'long_description': '# py-pdf-term\n\nA fully-configurable terminology extraction module written in Python\n\n## Installation\n\n```\npip install py-pdf-term\n```\n\nYou also need to install spaCy models `ja_core_news_sm` and `en_core_web_sm`, which this module depends on.\n\n```\npip install https://github.com/explosion/spacy-models/releases/download/ja_core_news_sm-3.2.0/ja_core_news_sm-3.2.0.tar.gz\npip install https://github.com/explosion/spacy-models/releases/download/en_core_web_sm-3.2.0/en_core_web_sm-3.2.0.tar.gz\n```\n\n## Documentation\n\nhttps://kumachan-mis.github.io/py-pdf-term\n',
    'author': 'Yuya Suwa',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kumachan-mis/py-pdf-term',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from dataclasses import dataclass, field
from typing import List

from .base import BaseLayerConfig


@dataclass(frozen=True)
class CandidateLayerConfig(BaseLayerConfig):
    lang_tokenizers: List[str] = field(
        default_factory=lambda: [
            "py_pdf_term.JapaneseTokenizer",
            "py_pdf_term.EnglishTokenizer",
        ]
    )
    token_classifiers: List[str] = field(
        default_factory=lambda: [
            "py_pdf_term.JapaneseTokenClassifier",
            "py_pdf_term.EnglishTokenClassifier",
        ]
    )
    token_filters: List[str] = field(
        default_factory=lambda: [
            "py_pdf_term.JapaneseTokenFilter",
            "py_pdf_term.EnglishTokenFilter",
        ]
    )
    term_filters: List[str] = field(
        default_factory=lambda: [
            "py_pdf_term.JapaneseConcatenationFilter",
            "py_pdf_term.EnglishConcatenationFilter",
            "py_pdf_term.JapaneseSymbolLikeFilter",
            "py_pdf_term.EnglishSymbolLikeFilter",
            "py_pdf_term.JapaneseProperNounFilter",
            "py_pdf_term.EnglishProperNounFilter",
            "py_pdf_term.JapaneseNumericFilter",
            "py_pdf_term.EnglishNumericFilter",
        ]
    )
    splitters: List[str] = field(
        default_factory=lambda: [
            "py_pdf_term.SymbolNameSplitter",
            "py_pdf_term.RepeatSplitter",
        ]
    )
    augmenters: List[str] = field(
        default_factory=lambda: [
            "py_pdf_term.JapaneseConnectorTermAugmenter",
            "py_pdf_term.EnglishConnectorTermAugmenter",
        ]
    )
    cache: str = "py_pdf_term.CandidateLayerFileCache"



from typing import Any, Callable, Dict, Iterable

from .base import BaseMultiDomainRankingMethod
from .collectors import MDPRankingDataCollector
from .rankers import MDPRanker
from .rankingdata import MDPRankingData


class MDPMethod(BaseMultiDomainRankingMethod[MDPRankingData]):
    def __init__(
        self, compile_scores: Callable[[Iterable[float]], float] = min
    ) -> None:
        collector = MDPRankingDataCollector()
        ranker = MDPRanker(compile_scores=compile_scores)
        super().__init__(collector, ranker)

    @classmethod
    def collect_data_from_dict(cls, obj: Dict[str, Any]) -> MDPRankingData:
        return MDPRankingData(**obj)



from py_pdf_term.tokenizer import Token
from .base import BaseTokenClassifier


class EnglishTokenClassifier(BaseTokenClassifier):
    def inscope(self, token: Token) -> bool:
        return token.lang == "en"

    def is_symbol(self, token: Token) -> bool:
        return token.pos in {"SYM"}

    def is_connector_symbol(self, token: Token) -> bool:
        return token.surface_form == "-" and token.pos == "SYM"

    def is_connector_term(self, token: Token) -> bool:
        return token.pos == "ADP" and token.category == "IN"



from py_pdf_term._common.data import Term
from py_pdf_term.tokenizer import Token
from ....classifiers import JapaneseTokenClassifier

from ..base import BaseJapaneseCandidateTermFilter


class JapaneseProperNounFilter(BaseJapaneseCandidateTermFilter):
    def __init__(self) -> None:
        self._classifier = JapaneseTokenClassifier()

    def is_candidate(self, scoped_term: Term) -> bool:
        return not self._is_region_or_person(scoped_term)

    def _is_region_or_person(self, scoped_term: Term) -> bool:
        def is_region_or_person_token(token: Token) -> bool:
            return (
                (
                    token.pos == "名詞"
                    and token.category == "固有名詞"
                    and token.subcategory in {"人名", "地名"}
                )
                or self._classifier.is_connector_term(token)
                or self._classifier.is_connector_symbol(token)
            )

        return all(map(is_region_or_person_token, scoped_term.tokens))



from .data import DomainTechnicalTermList, PageTechnicalTermList, PDFTechnicalTermList
from .extractor import TechnicalTermExtractor

# isort: unique-list
__all__ = [
    "DomainTechnicalTermList",
    "PDFTechnicalTermList",
    "PageTechnicalTermList",
    "TechnicalTermExtractor",
]



from dataclasses import dataclass
from typing import Any, Dict, List

from py_pdf_term._common.data import ScoredTerm


@dataclass(frozen=True)
class MethodTermRanking:
    domain: str
    ranking: List[ScoredTerm]
    # list of pairs of lemmatized term and method score
    # the list is sorted by the score in descending order

    def to_dict(self) -> Dict[str, Any]:
        return {
            "domain": self.domain,
            "ranking": list(map(lambda term: term.to_dict(), self.ranking)),
        }

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> "MethodTermRanking":
        return cls(
            obj["domain"],
            list(map(lambda item: ScoredTerm.from_dict(item), obj["ranking"])),
        )



from typing import Dict

from py_pdf_term._common.data import Term
from py_pdf_term._common.extended_math import extended_log10
from py_pdf_term.candidates import PageCandidateTermList

from .base import BaseStylingScore


class ColorScore(BaseStylingScore):
    def __init__(self, page_candidates: PageCandidateTermList) -> None:
        super().__init__(page_candidates)

        self._num_candidates = len(page_candidates.candidates)

        self._color_freq: Dict[str, int] = dict()
        for candidate in page_candidates.candidates:
            self._color_freq[candidate.ncolor] = (
                self._color_freq.get(candidate.ncolor, 0) + 1
            )

    def calculate_score(self, candidate: Term) -> float:
        if self._num_candidates == 0 or candidate.ncolor not in self._color_freq:
            return 1.0

        ncolor_prob = self._color_freq.get(candidate.ncolor, 0) / self._num_candidates
        return -extended_log10(ncolor_prob) + 1.0



from .base import BaseMultiDomainRanker, BaseSingleDomainRanker
from .flr import FLRRanker
from .flrh import FLRHRanker
from .hits import HITSRanker
from .mcvalue import MCValueRanker
from .mdp import MDPRanker
from .tfidf import TFIDFRanker

# isort: unique-list
__all__ = [
    "BaseMultiDomainRanker",
    "BaseSingleDomainRanker",
    "FLRHRanker",
    "FLRRanker",
    "HITSRanker",
    "MCValueRanker",
    "MDPRanker",
    "TFIDFRanker",
]



from dataclasses import dataclass, field
from typing import Dict

from .base import BaseRankingData


@dataclass(frozen=True)
class MDPRankingData(BaseRankingData):
    domain: str
    # unique domain name
    term_freq: Dict[str, int]
    # brute force counting of lemmatized term occurrences in the domain
    # count even if the lemmatized term occurs as a part of a lemmatized phrase
    num_terms: int = field(init=False)
    # brute force counting of all lemmatized terms occurrences in the domain
    # count even if the lemmatized term occurs as a part of a lemmatized phrase

    def __post_init__(self) -> None:
        object.__setattr__(self, "num_terms", sum(self.term_freq.values()))



import re
from abc import ABCMeta, abstractmethod

from py_pdf_term._common.consts import ENGLISH_REGEX, JAPANESE_REGEX, NUMBER_REGEX
from py_pdf_term._common.data import Term


class BaseCandidateTermFilter(metaclass=ABCMeta):
    def __init__(self) -> None:
        pass

    @abstractmethod
    def inscope(self, term: Term) -> bool:
        raise NotImplementedError(f"{self.__class__.__name__}.inscope()")

    @abstractmethod
    def is_candidate(self, scoped_term: Term) -> bool:
        raise NotImplementedError(f"{self.__class__.__name__}.is_candidate()")


class BaseJapaneseCandidateTermFilter(BaseCandidateTermFilter):
    def inscope(self, term: Term) -> bool:
        regex = re.compile(rf"({ENGLISH_REGEX}|{JAPANESE_REGEX}|{NUMBER_REGEX}|\s|\-)+")
        return term.lang == "ja" and regex.fullmatch(str(term)) is not None


class BaseEnglishCandidateTermFilter(BaseCandidateTermFilter):
    def inscope(self, term: Term) -> bool:
        regex = re.compile(rf"({ENGLISH_REGEX}|{NUMBER_REGEX}|\s|\-)+")
        return term.lang == "en" and regex.fullmatch(str(term)) is not None



from .base import BaseCandidateTokenFilter
from .english import EnglishTokenFilter
from .japanese import JapaneseTokenFilter

# isort: unique-list
__all__ = ["BaseCandidateTokenFilter", "EnglishTokenFilter", "JapaneseTokenFilter"]



from math import exp
from statistics import mean, stdev

from py_pdf_term._common.data import Term
from py_pdf_term.candidates import PageCandidateTermList

from .base import BaseStylingScore


class FontsizeScore(BaseStylingScore):
    def __init__(self, page_candidates: PageCandidateTermList) -> None:
        super().__init__(page_candidates)

        self._num_candidates = len(page_candidates.candidates)

        self._fontsize_mean = (
            mean(map(lambda candidate: candidate.fontsize, page_candidates.candidates))
            if self._num_candidates >= 1
            else None
        )
        self._fontsize_stdev = (
            stdev(
                map(lambda candidate: candidate.fontsize, page_candidates.candidates),
                self._fontsize_mean,
            )
            if self._num_candidates >= 2
            else None
        )

    def calculate_score(self, candidate: Term) -> float:
        if self._fontsize_mean is None or self._fontsize_stdev is None:
            return 1.0
        if self._fontsize_stdev == 0:
            return 1.0

        z = (candidate.fontsize - self._fontsize_mean) / self._fontsize_stdev
        return 2 / (1 + exp(-z))



from dataclasses import asdict, dataclass
from typing import Any, Dict, List

from py_pdf_term._common.data import ScoredTerm


@dataclass(frozen=True)
class PageStylingScoreList:
    page_num: int
    ranking: List[ScoredTerm]
    # list of pairs of lemmatized term and styling score
    # the list is sorted by the score in descending order

    def to_dict(self) -> Dict[str, Any]:
        return asdict(self)

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> "PageStylingScoreList":
        page_num, ranking = obj["page_num"], obj["ranking"]
        return cls(
            page_num,
            list(map(lambda item: ScoredTerm.from_dict(item), ranking)),
        )


@dataclass(frozen=True)
class PDFStylingScoreList:
    pdf_path: str
    pages: List[PageStylingScoreList]

    def to_dict(self) -> Dict[str, Any]:
        return {
            "pdf_path": self.pdf_path,
            "pages": list(map(lambda page: page.to_dict(), self.pages)),
        }

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> "PDFStylingScoreList":
        pdf_path, pages = obj["pdf_path"], obj["pages"]
        return cls(
            pdf_path,
            list(map(lambda item: PageStylingScoreList.from_dict(item), pages)),
        )


@dataclass(frozen=True)
class DomainStylingScoreList:
    domain: str
    pdfs: List[PDFStylingScoreList]

    def to_dict(self) -> Dict[str, Any]:
        return {
            "domain": self.domain,
            "pdfs": list(map(lambda pdf: pdf.to_dict(), self.pdfs)),
        }

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> "DomainStylingScoreList":
        domain, pdfs = obj["domain"], obj["pdfs"]
        return cls(
            domain,
            list(map(lambda item: PDFStylingScoreList.from_dict(item), pdfs)),
        )



from __future__ import annotations

from abc import ABCMeta, abstractmethod
from typing import Dict, Generic, List, TypeVar, Union

MappedValue = TypeVar("MappedValue")


class BaseMapper(Generic[MappedValue], metaclass=ABCMeta):
    def __init__(self) -> None:
        self._map: Dict[str, MappedValue] = dict()

    def add(self, name: str, value: MappedValue) -> None:
        self._map[name] = value

    def remove(self, name: str) -> None:
        del self._map[name]

    def find(self, name: str) -> MappedValue:
        return self._map[name]

    def find_or_none(self, name: str) -> Union[MappedValue, None]:
        return self._map.get(name)

    def bulk_find(self, names: List[str]) -> List[MappedValue]:
        return list(map(lambda name: self._map[name], names))

    def bulk_find_or_none(self, names: List[str]) -> List[Union[MappedValue, None]]:
        return list(map(self._map.get, names))

    @classmethod
    @abstractmethod
    def default_mapper(cls) -> BaseMapper[MappedValue]:
        raise NotImplementedError(f"{cls.__name__}.default_mapper()")



from .data import Token
from .tokenizer import Tokenizer
from .base import BaseLanguageTokenizer
from .english import EnglishTokenizer
from .japanese import JapaneseTokenizer

# isort: unique-list
__all__ = [
    "BaseLanguageTokenizer",
    "EnglishTokenizer",
    "JapaneseTokenizer",
    "Token",
    "Tokenizer",
]



import re

from py_pdf_term._common.consts import ALPHABET_REGEX, NUMBER_REGEX
from py_pdf_term._common.data import Term
from py_pdf_term.tokenizer import Token
from ....classifiers import EnglishTokenClassifier

from ..base import BaseEnglishCandidateTermFilter

PHONETIC_REGEX = ALPHABET_REGEX


class EnglishSymbolLikeFilter(BaseEnglishCandidateTermFilter):
    def __init__(self) -> None:
        self._classifier = EnglishTokenClassifier()
        self._phonetic_regex = re.compile(PHONETIC_REGEX)
        self._indexed_phonetic_regex = re.compile(
            rf"({PHONETIC_REGEX}{NUMBER_REGEX}+)+{PHONETIC_REGEX}?"
            + "|"
            + rf"({NUMBER_REGEX}+{PHONETIC_REGEX})+({NUMBER_REGEX}+)?"
        )

    def is_candidate(self, scoped_term: Term) -> bool:
        return (
            not self._is_phonetic_or_meaningless_term(scoped_term)
            and not self._is_indexed_phonetic(scoped_term)
            and not self._phonetic_token_appears_continuously(scoped_term)
        )

    def _is_phonetic_or_meaningless_term(self, scoped_term: Term) -> bool:
        def is_phonetic_or_meaningless_token(token: Token) -> bool:
            is_phonetic = self._phonetic_regex.fullmatch(str(token)) is not None
            is_meaningless = self._classifier.is_meaningless(token)
            return is_phonetic or is_meaningless

        return all(map(is_phonetic_or_meaningless_token, scoped_term.tokens))

    def _is_indexed_phonetic(self, scoped_term: Term) -> bool:
        return self._indexed_phonetic_regex.fullmatch(str(scoped_term)) is not None

    def _phonetic_token_appears_continuously(self, scoped_term: Term) -> bool:
        num_tokens = len(scoped_term.tokens)

        def phonetic_token_appears_continuously_at(i: int) -> bool:
            if i == num_tokens - 1:
                return False

            token_str = str(scoped_term.tokens[i])
            next_token_str = str(scoped_term.tokens[i + 1])
            return (
                self._phonetic_regex.fullmatch(token_str) is not None
                and self._phonetic_regex.fullmatch(next_token_str) is not None
            )

        return any(map(phonetic_token_appears_continuously_at, range(num_tokens)))



from dataclasses import dataclass
from typing import Dict

from .base import BaseRankingData


@dataclass(frozen=True)
class HITSRankingData(BaseRankingData):
    domain: str
    # unique domain name
    term_freq: Dict[str, int]
    # brute force counting of lemmatized term occurrences in the domain
    # count even if the lemmatized term occurs as a part of a lemmatized phrase
    left_freq: Dict[str, Dict[str, int]]
    # number of occurrences of lemmatized (left, token) in the domain
    # if token or left is meaningless this is fixed at zero
    right_freq: Dict[str, Dict[str, int]]
    # number of occurrences of lemmatized (token, right) in the domain
    # if token or right is meaningless this is fixed at zero



import json
from hashlib import sha256
from typing import List

from ..configs import BaseLayerConfig


def create_dir_name_from_config(config: BaseLayerConfig, prefix: str = "") -> str:
    config_class_name = config.__class__.__name__
    config_json = json.dumps(config.to_dict_without_cache())
    return f"{prefix}{sha256((config_class_name + config_json).encode()).hexdigest()}"


def create_file_name_from_path(path: str, ext: str, prefix: str = "") -> str:
    return f"{prefix}{sha256(path.encode()).hexdigest()}.{ext}"


def create_file_name_from_paths(paths: List[str], ext: str, prefix: str = "") -> str:
    return f"{prefix}{sha256(json.dumps(sorted(paths)).encode()).hexdigest()}.{ext}"



from .base import BaseXMLLayerCache
from .file import XMLLayerFileCache
from .nocache import XMLLayerNoCache

# isort: unique-list
__all__ = ["BaseXMLLayerCache", "XMLLayerFileCache", "XMLLayerNoCache"]



# http://www.unicode.org/charts/

# Alphabet
ALPHABET_REGEX = r"[A-Za-z]"

# Number
NUMBER_REGEX = r"[0-9]"

# Hiragana
HIRAGANA_REGEX = r"[\u3040-\u309F]"

# Katakana
KATAKANA_REGEX = r"[\u30A0-\u30FF]"

# Kanji
KANJI_REGEX = (
    "["
    + "".join(
        [
            # CJK Radicals Supplement
            "\u2E80-\u2EFF",
            # Kangxi Radicals
            "\u2F00-\u2FDF",
            # CJK Unified Ideographs Extension A
            "\u3400-\u4DBF",
            # CJK Unified Ideographs
            "\u4E00-\u9FFF",
            # CJK Compatibility Ideographs
            "\uF900-\uFAFF",
        ]
    )
    + "]"
)

# Symbol
SYMBOL_REGEX = (
    "["
    + "".join(
        [
            # Basic Latin Symbol
            "\u0021-\u002F",
            "\u003A-\u0040",
            "\u005B-\u0060",
            "\u007B-\u007F",
            # Latin-1 Supplement Symbol
            "\u00A1-\u00BF\u00D7\u00F7",
            # Spacing Modifier Letters - Combining Diacritical Marks
            "\u02B0-\u036F",
            # Phonetic Extensions - Combining Diacritical Marks Supplement
            "\u1D00-\u1DFF",
            # General Punctuation - Miscellaneous Symbols and Arrows
            "\u2010-\u2BFF",
            # Supplemental Punctuation
            "\u2E00-\u2E7F",
            # Ideographic Description Characters - CJK Symbols and Punctuation
            "\u2FF0-\u303F"
            # Bopomofo - CJK Strokes
            "\u3100-\u31EF",
            # Enclosed CJK Letters and Months - CJK Compatibility
            "\u3200-\u33FF",
            # Yijing Hexagram Symbols
            "\u4DC0-\u4DFF",
            # Modifier Tone Letters
            "\uA700-\uA71F",
            # High Surrogates - Private Use Area
            "\uD800-\uF8FF",
            # Variation Selectors - Small Form Variants
            "\uFE00-\uFE6F",
            # Specials
            "\uFFF0-\uFFFF",
        ]
    )
    + "]"
)

# Halfwidth and Fullwidth
HALFWIDTH_ASCII_CHARS = "".join([chr(code) for code in range(0x0021, 0x007E + 1)])
FULLWIDTH_ASCII_CHARS = "".join([chr(code) for code in range(0xFF01, 0xFF5E + 1)])

# Languages
JAPANESE_REGEX = rf"(?:{HIRAGANA_REGEX}|{KATAKANA_REGEX}|{KANJI_REGEX})"
ENGLISH_REGEX = rf"(?:{ALPHABET_REGEX})"

# Character requiring non-space between
NOSPACE_REGEX = (
    "(?:"
    + "|".join([HIRAGANA_REGEX, KATAKANA_REGEX, KANJI_REGEX, NUMBER_REGEX, "-"])
    + ")"
)



import os
from glob import glob
from shutil import rmtree
from typing import Union
from xml.etree.ElementTree import ParseError, fromstring, tostring

from py_pdf_term.pdftoxml import PDFnXMLElement

from ...configs import XMLLayerConfig
from ..util import create_dir_name_from_config, create_file_name_from_path
from .base import BaseXMLLayerCache


class XMLLayerFileCache(BaseXMLLayerCache):
    def __init__(self, cache_dir: str) -> None:

        self._cache_dir = cache_dir

    def load(
        self, pdf_path: str, config: XMLLayerConfig
    ) -> Union[PDFnXMLElement, None]:
        dir_name = create_dir_name_from_config(config)
        file_name = create_file_name_from_path(pdf_path, "xml")
        cache_file_path = os.path.join(self._cache_dir, dir_name, file_name)

        if not os.path.isfile(cache_file_path):
            return None

        with open(cache_file_path, "r") as xml_file:
            try:
                xml_root = fromstring(xml_file.read())
            except ParseError:
                return None

        return PDFnXMLElement(pdf_path, xml_root)

    def store(self, pdfnxml: PDFnXMLElement, config: XMLLayerConfig) -> None:
        dir_name = create_dir_name_from_config(config)
        file_name = create_file_name_from_path(pdfnxml.pdf_path, "xml")
        cache_file_path = os.path.join(self._cache_dir, dir_name, file_name)

        os.makedirs(os.path.dirname(cache_file_path), exist_ok=True)

        with open(cache_file_path, "wb") as xml_file:
            xml_content = tostring(pdfnxml.xml_root, encoding="utf-8")
            xml_file.write(xml_content)

    def remove(self, pdf_path: str, config: XMLLayerConfig) -> None:
        dir_name = create_dir_name_from_config(config)
        file_name = create_file_name_from_path(pdf_path, "xml")
        cache_dir_path = os.path.join(self._cache_dir, dir_name)
        cache_file_path = os.path.join(cache_dir_path, file_name)

        if not os.path.isfile(cache_file_path):
            return

        os.remove(cache_file_path)

        cache_file_paths = glob(os.path.join(cache_dir_path, "*.xml"))
        if not cache_file_paths:
            rmtree(cache_dir_path)



from dataclasses import dataclass
from typing import Any, Dict, Set

from .base import BaseRankingData


@dataclass(frozen=True)
class MCValueRankingData(BaseRankingData):
    domain: str
    # unique domain name
    term_freq: Dict[str, int]
    # brute force counting of lemmatized term occurrences in the domain
    # count even if the lemmatized term occurs as a part of a lemmatized phrase
    container_terms: Dict[str, Set[str]]
    # set of containers of the lemmatized term in the domain
    # (term, container) is valid iff the container contains the term
    # as a proper subsequence

    def to_dict(self) -> Dict[str, Any]:
        container_terms = {
            term: list(containers) for term, containers in self.container_terms.items()
        }
        return {
            "domain": self.domain,
            "term_freq": self.term_freq,
            "container_terms": container_terms,
        }

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> "MCValueRankingData":
        container_terms = {
            term: set(containers) for term, containers in obj["container_terms"].items()
        }
        return MCValueRankingData(obj["domain"], obj["term_freq"], container_terms)



from dataclasses import asdict, dataclass
from typing import Any, Dict
from xml.etree.ElementTree import Element, fromstring, tostring


@dataclass(frozen=True)
class PDFnXMLPath:
    pdf_path: str
    xml_path: str

    def to_dict(self) -> Dict[str, Any]:
        return asdict(self)

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> "PDFnXMLPath":
        return cls(**obj)


@dataclass(frozen=True)
class PDFnXMLElement:
    pdf_path: str
    xml_root: Element

    def to_dict(self) -> Dict[str, Any]:
        return {
            "pdf_path": self.pdf_path,
            "xml_root": tostring(self.xml_root, encoding="utf-8").decode("utf-8"),
        }

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> "PDFnXMLElement":
        return cls(obj["pdf_path"], fromstring(obj["xml_root"]))



from math import log, log2, log10
from typing import SupportsFloat


def extended_log(x: SupportsFloat, base: SupportsFloat) -> float:
    float_x = float(x)
    if float_x > 0.0:
        return log(float_x + 1.0, base)
    if float_x < 0.0:
        return -log(-float_x + 1.0, base)
    else:
        return 0.0


def extended_log2(__x: SupportsFloat) -> float:
    float_x = float(__x)
    if float_x > 0.0:
        return log2(float_x + 1.0)
    if float_x < 0.0:
        return -log2(-float_x + 1.0)
    else:
        return 0.0


def extended_log10(__x: SupportsFloat) -> float:
    float_x = float(__x)
    if float_x > 0.0:
        return log10(float_x + 1.0)
    if float_x < 0.0:
        return -log10(-float_x + 1.0)
    else:
        return 0.0



from typing import List, Optional

from py_pdf_term._common.data import Term

from .base import BaseSplitter
from .repeat import RepeatSplitter
from .symname import SymbolNameSplitter


class SplitterCombiner:
    def __init__(self, splitters: Optional[List[BaseSplitter]] = None) -> None:
        if splitters is None:
            splitters = [SymbolNameSplitter(), RepeatSplitter()]

        self._splitters = splitters

    def split(self, term: Term) -> List[Term]:
        splitted_terms = [term]

        for splitter in self._splitters:
            start: List[Term] = []
            splitted_terms = sum(map(splitter.split, splitted_terms), start)

        return splitted_terms



from .base import BaseStylingLayerCache
from .file import StylingLayerFileCache
from .nocache import StylingLayerNoCache

# isort: unique-list
__all__ = ["BaseStylingLayerCache", "StylingLayerFileCache", "StylingLayerNoCache"]
